breed [electrons electron]
globals [
  left-current right-current  ;; current measurements, accumulate for 100 ticks
  last-left-current last-right-current  ;; from previous 100 tick period
]

;;;;;;;;;;;;;;;;;;;;;;;;
;;; Setup Procedures ;;;
;;;;;;;;;;;;;;;;;;;;;;;;

to setup
  clear-all
  set-default-shape electrons "circle 2"  ;; circle with a border
  ;; create left wire
  ask patches with [pxcor < 0]
    [ set pcolor gray ]
  ;; create the right wire
  ask patches with [pxcor >= 0]
    [ set pcolor gray + 3 ]  ;; light gray
  ;; set up battery negative
  ask patches with [pxcor >= max-pxcor - 4]
    [ set pcolor red ]
  ask patch (max-pxcor - 1) 0
    [ set plabel "-" ]
  ;; set up battery positive
  ask patches with [pxcor <= min-pxcor + 4]
    [ set pcolor black ]
  ask patch (min-pxcor + 4) 0
    [ set plabel "+" ]
  ;; create white strip between two wires for calculating current out of the right-wire
  ask patches with [pxcor < 1 and pxcor > -1]
    [ set pcolor white ]
  ;; make sure that no electrons are on the Battery-positive - move them over
  ;; to the battery-negative by making them wrap around
  ask electrons with [pcolor = black] [ set xcor xcor - 7 ]
  ask electrons with [pcolor = white] [ set xcor xcor - 2 ]
  ;; now create electrons on the wires. we use the collision rates
  ;; to determine the initial densities, so that the densities
  ;; won't change (much) over the course of the run
  ;; create electrons in left wire
  ask n-of (600 * collision-rate-with-atoms-left / normalization-factor)
           patches with [pcolor = gray + 3]
    [ sprout-electron ]
  ;; create electrons in right wire
  ask n-of (600 * collision-rate-with-atoms-right / normalization-factor)
           patches with [pcolor = gray ]
    [ sprout-electron ]
end

to sprout-electron  ;; patch procedure
  sprout-electrons 1
  [
    set color orange - 2  ;; dark orange
    set size 1.5
  ]
end

;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Runtime Procedures ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;

to go
  ask electrons
  [
    ifelse pcolor = gray + 3
      [ move-electron collision-rate-with-atoms-right ]
      [ if pcolor = gray
          [ move-electron collision-rate-with-atoms-left ] ]
    ;; if electrons are on the battery-negative, move them into the right wire
    ;; with the same speed that the electrons have inside the right wire
    if pcolor = red
      [ set xcor xcor - Voltage * collision-rate-with-atoms-right ]
    ;; count number of electrons arriving at the battery-positive per tick
    if pcolor = black [
      set left-current left-current + 1
      set xcor xcor - 6
    ]
    ;; count number of electrons leaving the right-wire and entering the left-wire per tick
    if pcolor = white
      [ set right-current right-current + 1
        set xcor xcor - 2 ]
  ]
  tick
  if ticks mod 100 = 0 [ update-plots ]
end

;; wiggle randomly, performing simple point collisions
;; with invisible nuclei in the wire and steadily drifting
;; forward due to the electric field
to move-electron [collision-rate]
  ;; perform collisions with (invisible) atoms in the wire
  if random-float 1.0 < collision-rate
    [ rt random 360 ]
  fd 0.3
  ;; drift due to electric field towards the battery positive
  set xcor xcor - voltage * collision-rate / normalization-factor
end

;; normalization factor for calculating the effective voltage
;; and number of electrons in each wire
to-report normalization-factor
  report collision-rate-with-atoms-left + collision-rate-with-atoms-right
end

;;;;;;;;;;;;;;;;;;;;;;;
;; Plotting procedures
;;;;;;;;;;;;;;;;;;;;;;;

to update-plots
  set-current-plot "Current in right wire"
  plotxy (ticks - 100) (right-current / 100)
  set last-right-current right-current / 100
  set right-current 0
  set-current-plot "Current in left wire"
  plotxy (ticks - 100) (left-current / 100)
  set last-left-current left-current / 100
  set left-current 0
end


; *** NetLogo 4.0.4 Model Copyright Notice ***
;
; Copyright 2007 by Uri Wilensky.  All rights reserved.
;
; Permission to use, modify or redistribute this model is hereby granted,
; provided that both of the following requirements are followed:
; a) this copyright notice is included.
; b) this model will not be redistributed for profit without permission
;    from Uri Wilensky.
; Contact Uri Wilensky for appropriate licenses for redistribution for
; profit.
;
; To refer to this model in academic publications, please use:
; Sengupta, P. and Wilensky, U. (2007). NetLogo Series Circuit model.
; http://ccl.northwestern.edu/netlogo/models/SeriesCircuit.
; Center for Connected Learning and Computer-Based Modeling,
; Northwestern University, Evanston, IL.
;
; In other publications, please use:
; Copyright 2007 Uri Wilensky.  All rights reserved.
; See http://ccl.northwestern.edu/netlogo/models/SeriesCircuit
; for terms of use.
;
; *** End of NetLogo 4.0.4 Model Copyright Notice ***
@#$#@#$#@
GRAPHICS-WINDOW
223
10
917
173
85
16
4.0
1
30
1
1
1
0
1
1
1
-85
85
-16
16
1
1
1
ticks

CC-WINDOW
5
390
930
485
Command Center
0

BUTTON
8
21
92
61
NIL
setup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL

BUTTON
117
20
201
61
NIL
go
T
1
T
OBSERVER
NIL
NIL
NIL
NIL

SLIDER
10
72
199
105
voltage
voltage
0
1.5
1
0.1
1
NIL
HORIZONTAL

PLOT
287
209
556
330
Current in left wire
time
current
0.0
0.0
0.0
0.35
true
false
PENS
"default" 1.0 0 -16777216 true

TEXTBOX
224
179
312
204
Battery-positive
11
0.0
0

TEXTBOX
829
178
921
206
Battery-negative\n
11
0.0
0

SLIDER
330
173
556
206
collision-rate-with-atoms-left
collision-rate-with-atoms-left
0.3
2
2
0.1
1
NIL
HORIZONTAL

SLIDER
587
173
809
206
collision-rate-with-atoms-right
collision-rate-with-atoms-right
0.5
1
0.5
0.1
1
NIL
HORIZONTAL

PLOT
587
209
858
332
Current in right wire
time
current
0.0
0.0
0.0
0.35
true
false
PENS
"default" 1.0 0 -16777216 true

MONITOR
587
331
669
376
# electrons
count electrons with [pcolor = gray + 3]
0
1
11

MONITOR
287
329
374
374
# electrons
count electrons with [pcolor = gray]
3
1
11

MONITOR
668
331
734
376
Current
last-right-current
2
1
11

MONITOR
373
329
435
374
Current
last-left-current
2
1
11

BUTTON
9
111
200
144
trace and watch a single electron
ask one-of electrons \n  [ set shape \"circle\" \n    set color blue - 1\n    pd\n    watch-me\n  ]
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL

BUTTON
9
148
200
181
watch a single electron
watch one-of electrons
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL

BUTTON
9
187
201
220
stop watching and tracing
reset-perspective\nask turtles \n[\n  set color red - 2\n  pu\n  ]\n
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL

MONITOR
434
329
556
374
Voltage (left wire)
Voltage\n  * collision-rate-with-atoms-left\n  / normalization-factor
3
1
11

MONITOR
733
331
858
376
Voltage (right wire)
Voltage\n  * collision-rate-with-atoms-right\n  / normalization-factor
3
1
11

@#$#@#$#@
WHAT IS IT?
-----------
This model shows a microscopic view of free electrons in two conducting wires that are connected in series between two terminals of a battery.

This model shows what occurs when two wires are connected in series between two ends of a battery terminal.  It illustrates how current in one wire is always equal to current in the other, even when the wires have different resistances.   It also displays how changing resistance in a single wire affects the magnitude of current in both wires.


HOW IT WORKS
------------
The rules in this model are the same as in the earlier Ohm's Law model, except that instead of the collision rate being uniform throughout the wire, the wire is divided into two regions connected end to end, each with its own collision rate. Each segment acts as a separate wire.


HOW TO USE IT
-------------
The TOTAL-ELECTRONS slider allows you to select the total number of free electrons in both the wires. This number is maintained constant throughout a single run of the model.

The VOLTAGE slider imparts a steady velocity to the electrons.  However, this velocity is also inversely proportional to the rate at which the electrons collide with the atomic nuclei in the wires.

The COLLISION-RATE-WITH-NUCLEI sliders, one for each wire, are inversely proportional to how far an electron travels on average without colliding with atomic nuclei. The collision rate of electrons in a wire causes resistance. The collision-rate affects the motion of electrons in it in another way: the net velocity of the electrons is also reduced in proportion to the collision rate.

The TRACE AND WATCH A SINGLE ELECTRON button highlights an electron and traces its path.  The WATCH A SINGLE ELECTRON button is the same, but without the trace.  Press STOP WATCHING AND TRACING to remove the highlighting.  (To clear traces, start over by pressing SETUP.)


THINGS TO NOTICE
----------------
When you observe the trace of the path of an electron, how does it differ in the two wires?  Why?

What happens to the number of electrons in each wire when you change the collision rate of electrons in either of the wires?


THINGS TO TRY
-------------
1. Run the model with the default settings.  Observe the current in both the wires.  Are these values equal? What about the number of electrons in each wire?

2. Increase the collision-rate in one of the wires.  Note the current in both the wires. Is current in each wire still equal to each other?  What about the number of electrons in each wire?

3. Increase the collision rate in both the wires at the same time. Compare the current in both the wires.  Does the current in both wire sections remain equal ?  What about the number of electrons in each wire?

4. Press WATCH A SINGLE ELECTRON.  Using the tick counter, write down how many ticks of model time the electron takes to travel through each wire.  Repeat this observation several times for different values of the collision-rates in each wire.
a) What do you notice?
b) Given the total number of electrons in each wire and the length of the wires, how can you calculate current in each wire with the time an electron takes to travel through the wire?

5. Look in the section titled "Procedures for measuring current" in the Procedures tab. How is current in each wire calculated in this model?  Are this method and 3(b) equivalent?

6. How would you calculate the total current in the circuit?
Is it the same as current in each wire?
Or is it the sum of the two currents?
What are the reasons for your answers?


EXTENDING THE MODEL
-------------------
Can you divide the region between the two battery terminals into three wires (segments) instead of two?


NETLOGO FEATURES
----------------
Electrons wrap around the world both vertically and horizontally.


RELATED MODELS
--------------
Electrostatics
Ohm's Law
Parallel Circuit


CREDITS AND REFERENCES
----------------------
This model is a part of the NIELS curriculum. The NIELS curriculum is currently under development at Northwestern's Center for Connected Learning and Computer-Based Modeling. For more information about the NIELS curriculum please refer to http://ccl.northwestern.edu/NIELS.

Thanks to Daniel Kornhauser for his work on the design of this model.

To refer to this model in academic publications, please use:  Sengupta, P. and Wilensky, U. (2007).  NetLogo Series Circuit model.  http://ccl.northwestern.edu/netlogo/models/SeriesCircuit.  Center for Connected Learning and Computer-Based Modeling, Northwestern University, Evanston, IL.

In other publications, please use:  Copyright 2007 Uri Wilensky.  All rights reserved.  See http://ccl.northwestern.edu/netlogo/models/SeriesCircuit for terms of use.
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300
Circle -7500403 false true -45 -45 180
Circle -16777216 false false -2 -2 304

circle 2
false
0
Circle -16777216 true false 0 0 300
Circle -7500403 true true 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 4.0beta7
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
<experiments>
  <experiment name="experiment 1" repetitions="1" runMetricsEveryStep="true">
    <setup>setup</setup>
    <go>go</go>
    <timeLimit steps="1000"/>
    <metric>left-current / 1000</metric>
    <metric>right-current / 1000</metric>
    <enumeratedValueSet variable="total-electrons">
      <value value="300"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="collision-rate-with-nuclei-left">
      <value value="1"/>
      <value value="1.5"/>
      <value value="3"/>
      <value value="4"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="collision-rate-with-nuclei-right">
      <value value="2"/>
      <value value="2.5"/>
      <value value="3"/>
    </enumeratedValueSet>
  </experiment>
</experiments>
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 0.0 1.0
0.0 1 1.0 0.0
0.2 0 0.0 1.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180

@#$#@#$#@
